/************************************************************************
 *   IRC - Internet Relay Chat, server/s_lev.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "s_conf.h"
#include "h.h"

void sendto_lev(int lev, char *pattern, ...)
{

    char nbuf[512];
    va_list vl;
    dlink_node *ptr;
    aClient *cptr;

    va_start(vl, pattern);

    switch(lev)
    {

	case CCONN_LEV:
		for (ptr = snomask_client_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Connect", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case REJ_LEV:
		for (ptr = snomask_reject_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Reject", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case SKILL_LEV:
		for (ptr = snomask_kills_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Kill", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case SPAM_LEV:
	case FLOOD_LEV:
		for (ptr = snomask_flood_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Flood", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case DEBUG_LEV:
		for (ptr = snomask_debug_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Debug", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case SERVICE_LEV:
		for (ptr = snomask_snotice_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Services", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case SPY_LEV:
	case SNOTICE_LEV:
		for (ptr = snomask_snotice_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Server", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case PROXY_LEV:
		for (ptr = snomask_proxy_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Proxy", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case KLINE_LEV:
		for (ptr = snomask_bans_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Bans", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case QLINE_LEV:
		for (ptr = snomask_bans_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Q-Line", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	case ULINE_LEV:
		for (ptr = snomask_bans_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "U-Line", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

	default:
		for (ptr = snomask_snotice_list.head; ptr; ptr = ptr->next) {
		    cptr = ptr->data;
		    if (!cptr) continue;
		    ircsprintf(nbuf, ":%C %s %s :*** %s -- %s", &me, MSG_NOTICE, cptr->name, "Notice", pattern);
		    vsendto_one(cptr, nbuf, vl);
		}
		break;

    }

    va_end(vl);
    return;

}
